-------------------------------------------------------------------------------
-- File name : control.vhd
-- Title     : Control logic
-- project   : ECE 485
-- Library   : multi-cycle processor
-- Author(s) : Ivan Castellanos
-- Purpose   : definition of multi-cycle processor
-- notes :   

-- Copyright Illinois Institute of Technology
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity control_logic is
  port (irwrite      : in  std_logic;
        wb_ack       : in  std_logic;
        stb_o        : in  std_logic;
        overflow     : in  std_logic;
        mux_ctrl_sel : out std_logic_vector(1 downto 0));
end control_logic;

architecture struct of control_logic is

  signal ctrl_logic_inputs: std_logic_vector (3 downto 0);

begin
  ctrl_logic_inputs(3) <= stb_o;
  ctrl_logic_inputs(2) <= wb_ack;
  ctrl_logic_inputs(1) <= irwrite;
  ctrl_logic_inputs(0) <= overflow;
  
  process (ctrl_logic_inputs)
  begin
    case ctrl_logic_inputs(3 downto 0) is
      when "0000" => mux_ctrl_sel <= "00";	-- count
      when "0001" => mux_ctrl_sel <= "11";	-- overflow exception
      when "0010" => mux_ctrl_sel <= "00";
      when "0011" => mux_ctrl_sel <= "00";
      when "0100" => mux_ctrl_sel <= "00";	-- count
      when "0101" => mux_ctrl_sel <= "00";	-- count
      when "0110" => mux_ctrl_sel <= "00";
      when "0111" => mux_ctrl_sel <= "00";
      when "1000" => mux_ctrl_sel <= "10";	-- hold state (no ack) (mem reference)
      when "1001" => mux_ctrl_sel <= "10";	-- hold state (no ack) (mem reference)
      when "1010" => mux_ctrl_sel <= "10";	-- hold state (no ack) (if)
      when "1011" => mux_ctrl_sel <= "10";	-- hold state (no ack) (if)
      when "1100" => mux_ctrl_sel <= "00";	-- count
      when "1101" => mux_ctrl_sel <= "00";	-- count
      when "1110" => mux_ctrl_sel <= "01";	-- fetch rom addr
      when others => mux_ctrl_sel <= "01";	-- fetch rom addr
    end case;
    
  end process;

end struct;


