-------------------------------------------------------------------------------
-- File name : regisetrfile.vhd
-- Title     : 32 x 32-bit Register-file 
-- project   : ECE 485
-- Library   : multi-cycle processor
-- Author(s) : Ivan Castellanos
-- Purpose   : definition of multi-cycle processor
-- notes :   

-- Copyright Illinois Institute of Technology
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

use std.textio.all;
use IEEE.std_logic_textio.all;

entity registerfile is
port
(
   readreg1   : in  std_logic_vector(4 downto 0);
   readreg2   : in  std_logic_vector(4 downto 0);
   writereg  : in  std_logic_vector(4 downto 0);
   writedata : in  std_logic_vector(31 downto 0);
   regwrite  : in  std_logic;
   clk       : in  std_logic;
   rst       : in  std_logic;
   readdata1o : out std_logic_vector(31 downto 0);
   readdata2o : out std_logic_vector(31 downto 0)
);
end registerfile;

architecture behavior of registerfile is

--  component register32
--    port (d   : in  std_logic_vector(31 downto 0);
--          clk : in  std_logic;
--          q   : out std_logic_vector(31 downto 0));
--  end component;

component register32j
port
(
   d   : in  std_logic_vector(31 downto 0);
   clk : in  std_logic;
   rst : in  std_logic;
   init: in  std_logic_vector(31 downto 0);
   q   : out std_logic_vector(31 downto 0)
);
end component register32j;

component mux32_32 is
port
(
   I00,I01,I02,I03,I04,I05,I06,I07,
   I08,I09,I10,I11,I12,I13,I14,I15,
   I16,I17,I18,I19,I20,I21,I22,I23,
   I24,I25,I26,I27,I28,I29,I30,I31:   in std_logic_vector(31 downto 0);
   SL:                                in std_logic_vector(4 downto 0);
   R:                                 out std_logic_vector(31 downto 0)
);
end component mux32_32;


  -- write signals to individual registers
  signal wr1, wr2, wr3, wr4, wr5, wr6, wr7, wr8, wr9 : std_logic;
  signal wr10, wr11, wr12, wr13, wr14, wr15, wr16, wr17, wr18, wr19 : std_logic;
  signal wr20, wr21, wr22, wr23, wr24, wr25, wr26, wr27, wr28, wr29 : std_logic;
  signal wr30, wr31 : std_logic;

  -- outputs (contents) of each register
  signal q1, q2, q3, q4, q5, q6, q7, q8, q9 : std_logic_vector(31 downto 0);
  signal q10, q11, q12, q13, q14, q15, q16, q17, q18, q19 : std_logic_vector(31 downto 0);
  signal q20, q21, q22, q23, q24, q25, q26, q27, q28, q29 : std_logic_vector(31 downto 0);
  signal q30, q31: std_logic_vector(31 downto 0);

signal readdata1, readdata2: std_logic_vector(31 downto 0);
signal clkn: std_logic;

begin
    
  clkn <= not clk;
    
  reg1  : register32j port map(writedata, wr1, rst,  "00000000000000000000000000000001", q1);
  reg2  : register32j port map(writedata, wr2, rst,  "00000000000000000000000000000010", q2);
  reg3  : register32j port map(writedata, wr3, rst,  "00000000000000000000000000000011", q3);
  reg4  : register32j port map(writedata, wr4, rst,  "00000000000000000000000000000100", q4);
  reg5  : register32j port map(writedata, wr5, rst,  "00000000000000000000000000000101", q5);
  reg6  : register32j port map(writedata, wr6, rst,  "00000000000000000000000000000110", q6);
  reg7  : register32j port map(writedata, wr7, rst,  "00000000000000000000000000000111", q7);
  reg8  : register32j port map(writedata, wr8, rst,  "00000000000000000000000000001000", q8);
  reg9  : register32j port map(writedata, wr9, rst,  "00000000000000000000000000001001", q9);
  reg10 : register32j port map(writedata, wr10, rst, "00000000000000000000000000001010", q11);
  reg11 : register32j port map(writedata, wr11, rst, "00000000000000000000000000001011", q10);
  reg12 : register32j port map(writedata, wr12, rst, "00000000000000000000000000001100", q12);
  reg13 : register32j port map(writedata, wr13, rst, "00000000000000000000000000001101", q13);
  reg14 : register32j port map(writedata, wr14, rst, "00000000000000000000000000001110", q14);
  reg15 : register32j port map(writedata, wr15, rst, "00000000000000000000000000001111", q15);
  reg16 : register32j port map(writedata, wr16, rst, "00000000000000000000000000010000", q16);
  reg17 : register32j port map(writedata, wr17, rst, "00000000000000000000000000010001", q17);
  reg18 : register32j port map(writedata, wr18, rst, "00000000000000000000000000010010", q18);
  reg19 : register32j port map(writedata, wr19, rst, "00000000000000000000000000010011", q19);
  reg20 : register32j port map(writedata, wr20, rst, "00000000000000000000000000010100", q20);
  reg21 : register32j port map(writedata, wr21, rst, "00000000000000000000000000010101", q21);
  reg22 : register32j port map(writedata, wr22, rst, "00000000000000000000000000010110", q22);
  reg23 : register32j port map(writedata, wr23, rst, "00000000000000000000000000010111", q23);
  reg24 : register32j port map(writedata, wr24, rst, "00000000000000000000000000011000", q24);
  reg25 : register32j port map(writedata, wr25, rst, "00000000000000000000000000011001", q25);
  reg26 : register32j port map(writedata, wr26, rst, "00000000000000000000000000011010", q26);
  reg27 : register32j port map(writedata, wr27, rst, "00000000000000000000000000011011", q27);
  reg28 : register32j port map(writedata, wr28, rst, "00000000000000000000000000011100", q28);
  reg29 : register32j port map(writedata, wr29, rst, "00000000000000000000000000011101", q29);
  reg30 : register32j port map(writedata, wr30, rst, "00000000000000000000000000011110", q30);
  reg31 : register32j port map(writedata, wr31, rst, "00000000000000000000000000011111", q31);
  
  -- write register signals for each register (register clock inputs)
  wr1  <= not writereg(4) and not writereg( 3) and not writereg(2) and not writereg(1) and writereg(0) and regwrite and not clk;
  wr2  <= not writereg(4) and not writereg( 3) and not writereg(2) and writereg(1) and not writereg(0) and regwrite and not clk;
  wr3  <= not writereg(4) and not writereg( 3) and not writereg(2) and writereg(1) and writereg(0) and regwrite and not clk;
  wr4  <= not writereg(4) and not writereg( 3) and writereg(2) and not writereg(1) and not writereg(0) and regwrite and not clk;
  wr5  <= not writereg(4) and not writereg( 3) and writereg(2) and not writereg(1) and writereg(0) and regwrite and not clk;
  wr6  <= not writereg(4) and not writereg( 3) and writereg(2) and writereg(1) and not writereg(0) and regwrite and not clk;
  wr7  <= not writereg(4) and not writereg( 3) and writereg(2) and writereg(1) and writereg(0) and regwrite and not clk;
  wr8  <= not writereg(4) and writereg(3) and not writereg(2) and not writereg(1) and not writereg(0) and regwrite and not clk;
  wr9  <= not writereg(4) and writereg(3) and not writereg(2) and not writereg(1) and writereg(0) and regwrite and not clk;
  wr10 <= not writereg(4) and writereg(3) and not writereg(2) and writereg(1) and not writereg(0) and regwrite and not clk;
  wr11 <= not writereg(4) and writereg(3) and not writereg(2) and writereg(1) and writereg(0) and regwrite and not clk;
  wr12 <= not writereg(4) and writereg(3) and writereg(2) and not writereg(1) and not writereg(0) and regwrite and not clk;
  wr13 <= not writereg(4) and writereg(3) and writereg(2) and not writereg(1) and writereg(0) and regwrite and not clk;
  wr14 <= not writereg(4) and writereg(3) and writereg(2) and writereg(1) and not writereg(0) and regwrite and not clk;
  wr15 <= not writereg(4) and writereg(3) and writereg(2) and writereg(1) and writereg(0) and regwrite and not clk;
  wr16 <= writereg(4) and not writereg(3) and not writereg(2) and not writereg(1) and not writereg(0) and regwrite and not clk;
  wr17 <= writereg(4) and not writereg(3) and not writereg(2) and not writereg(1) and writereg(0) and regwrite and not clk;
  wr18 <= writereg(4) and not writereg(3) and not writereg(2) and writereg(1) and not writereg(0) and regwrite and not clk;
  wr19 <= writereg(4) and not writereg(3) and not writereg(2) and writereg(1) and writereg(0) and regwrite and not clk;
  wr20 <= writereg(4) and not writereg(3) and writereg(2) and not writereg(1) and not writereg(0) and regwrite and not clk;
  wr21 <= writereg(4) and not writereg(3) and writereg(2) and not writereg(1) and writereg(0) and regwrite and not clk;
  wr22 <= writereg(4) and not writereg(3) and writereg(2) and writereg(1) and not writereg(0) and regwrite and not clk;
  wr23 <= writereg(4) and not writereg(3) and writereg(2) and writereg(1) and writereg(0) and regwrite and not clk;
  wr24 <= writereg(4) and writereg(3) and not writereg(2) and not writereg(1) and not writereg(0) and regwrite and not clk;
  wr25 <= writereg(4) and writereg(3) and not writereg(2) and not writereg(1) and writereg(0) and regwrite and not clk;
  wr26 <= writereg(4) and writereg(3) and not writereg(2) and writereg(1) and not writereg(0) and regwrite and not clk;
  wr27 <= writereg(4) and writereg(3) and not writereg(2) and writereg(1) and writereg(0) and regwrite and not clk;
  wr28 <= writereg(4) and writereg(3) and writereg(2) and not writereg(1) and not writereg(0) and regwrite and not clk;
  wr29 <= writereg(4) and writereg(3) and writereg(2) and not writereg(1) and writereg(0) and regwrite and not clk;
  wr30 <= writereg(4) and writereg(3) and writereg(2) and writereg(1) and not writereg(0) and regwrite and not clk;
  wr31 <= writereg(4) and writereg(3) and writereg(2) and writereg(1) and writereg(0) and regwrite and not clk;

-- 2 32x32 muxes
RSMUX:   mux32_32 port map("00000000000000000000000000000000",q1,q2,q3,q4,q5,q6,q7,q8,q9,q10,q11,q12,q13,q14,q15,q16,q17,q18,q19,q20,q21,q22,q23,q24,q25,q26,q27,q28,q29,q30,q31,readreg1,readdata1);
RTMUX:   mux32_32 port map("00000000000000000000000000000000",q1,q2,q3,q4,q5,q6,q7,q8,q9,q10,q11,q12,q13,q14,q15,q16,q17,q18,q19,q20,q21,q22,q23,q24,q25,q26,q27,q28,q29,q30,q31,readreg2,readdata2);
  
----------

process(clkn) is
    variable l : line;
begin
    
if(clkn'event and clkn='1') then
  write(l, now);

  write(l, string'(" (RF) - R1("));
  write(l, readreg1);
  write(l, string'(")->"));
  write(l, readdata1);

  write(l, string'("; R2("));
  write(l, readreg2);
  write(l, string'(")->"));
  write(l, readdata2);

  write(l, string'("; R3("));
  write(l, writereg);
  write(l, string'(")<-"));
  write(l, writedata);

  write(l, string'(";  RegWrite="));
  write(l, regwrite);
  
  writeline(output, l);
end if;
end process;
----------

readdata1o <= readdata1;
readdata2o <= readdata2;
  
end behavior;

