-- Toplevel
-- Author: John Moskal
-- Modifications:
--
------------------------------------------------------------------------------------------

library work;
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;

---------------------------------------------------------------------------

entity toplevel is
end toplevel;

---------------------------------------------------------------------------

architecture struct of toplevel is

---------------------------------------------------------------------------
-- COMPONENT HERE

component datapath is
port
(
-- Common Control Signals
   C_Clk:         in std_logic;
   C_Rst:         in std_logic;
   
-- Flush
   C_FDFlush:     in std_logic;
   C_DEFLush:     in std_logic;
   C_EMFlush:     in std_logic;

-- Instuction Fetch Control Signals
   C_Exception:      in std_logic;
   C_Jump:      in std_logic;
   C_DBranch:   in std_logic;

-- Instruction Decode Control Signals
   C_BeqBne:      in std_logic;
   C_JmpType:     in std_logic;
   C_JalDet:      in std_logic;

-- Execute Control Signals
   C_AluOpCode:   in std_logic_vector(2 downto 0);
   C_DstRegMux:   in std_logic;
   C_Op2SrcMux:   in std_logic;

-- Memory Access Control Signals
   C_MemWrite:    in std_logic;

-- Write Back Control Signals
   C_MemAluMux:     in std_logic;
   C_RegWrite:    in std_logic;

-- Other Stuff
   C_EPCSrc:      in std_logic;
   C_EPCWrite:    in std_logic;
   C_Cause:       in std_logic;
   C_CauseWrite:  in std_logic;
   
-- ALU Output  
   E_AluCout:     out std_logic;
   E_AluZero:     out std_logic;

-- Exception Output
   O_EPCAdrOut:   out std_logic_vector(31 downto 0);
   O_CauseOut:    out std_logic;
   
-- Control Output
   CO_Opcode:     out std_logic_vector(5 downto 0);
   CO_Func:       out std_logic_vector(5 downto 0);
   CO_ALUOpcode:  out std_logic_vector(2 downto 0);
   CO_ALUOvf:     out std_logic
);
end component datapath;

component simplecontrol is
port
(
   rst:         in std_logic;
   opcode:       in std_logic_vector(5 downto 0);
   func:         in std_logic_vector(5 downto 0);
   ALUOpcode: in std_logic_vector(2 downto 0);
   ALUOvf:     in std_logic;
   
   C_FDFlush:    out std_logic;
   C_DEFlush:    out std_logic;
   C_EMFlush:    out std_logic;
   C_Exception:  out std_logic;
   C_Jump:       out std_logic;
   C_DBranch:    out std_logic;
   C_BeqBne:     out std_logic;
   C_JmpType:    out std_logic;
   C_JalDet:     out std_logic;
   C_ALUOpcode:  out std_logic_vector(2 downto 0);
   C_DstRegMux:  out std_logic;
   C_Op2SrcMux:  out std_logic;
   C_MemWrite:   out std_logic;
   C_MemAluMux:    out std_logic;
   C_RegWrite:   out std_logic;
   C_EPCSrc:     out std_logic;
   C_EPCWrite:   out std_logic;
   C_CauseWrite: out std_logic;
   C_Cause:      out std_logic
);
end component simplecontrol;

component clockgen is 
port
( 
num:   out integer; 
clk:   out std_logic;
rst:   out std_logic
); 
end component clockgen; 

---------------------------------------------------------------------------
-- SIGNALS HERE

signal   C_Clk:         std_logic;
signal   C_Rst:         std_logic;

signal   E_AluCout:     std_logic;
signal   E_AluZero:     std_logic;

signal   O_EPCAdrOut:   std_logic_vector(31 downto 0);
signal   O_CauseOut:    std_logic;

-- Control Signals
   
signal   C_FDFlush:     std_logic;
signal   C_DEFLush:     std_logic;
signal   C_EMFlush:     std_logic;
signal   C_Jump:        std_logic;
signal   C_DBranch:     std_logic;
signal   C_BeqBne:      std_logic;
signal   C_JmpType:     std_logic;
signal   C_JalDet:      std_logic;
signal   C_AluOpCode:   std_logic_vector(2 downto 0);
signal   C_DstRegMux:   std_logic;
signal   C_Op2SrcMux:   std_logic;
signal   C_MemWrite:    std_logic;
signal   C_MemAluMux:   std_logic;
signal   C_RegWrite:    std_logic;
signal   C_Exception:   std_logic;
signal   C_EPCSrc:      std_logic;
signal   C_EPCWrite:    std_logic;
signal   C_Cause:       std_logic;
signal   C_CauseWrite:  std_logic;

-- Output Control Signals
signal   CO_Opcode:     std_logic_vector(5 downto 0);
signal   CO_Func:       std_logic_vector(5 downto 0);
signal   CO_ALUOpcode:  std_logic_vector(2 downto 0);
signal   CO_ALUOvf:     std_logic;
   
-- Cycle Numbers
signal   CycleNum:      integer;
---------------------------------------------------------------------------

begin  -- struct

-- INSTANTIATE HERE
dtpth: datapath port map
(
C_Clk,
C_Rst,
C_FDFlush,
C_DEFLush,
C_EMFlush,
C_Exception,
C_Jump,
C_DBranch,
C_BeqBne,
C_JmpType,
C_JalDet,
C_AluOpCode,
C_DstRegMux,
C_Op2SrcMux,
C_MemWrite,
C_MemAluMux,
C_RegWrite,
C_EPCSrc,
C_EPCWrite,
C_Cause,
C_CauseWrite,
E_AluCout,
E_AluZero,
O_EPCAdrOut,
O_CauseOut,
CO_Opcode,
CO_Func,
CO_ALUOpcode,
CO_ALUOvf
);

cntrl: simplecontrol port map
(
C_Rst,
CO_Opcode,
CO_Func,
CO_ALUOpcode,
CO_ALUOvf,
C_FDFlush,
C_DEFlush,
C_EMFlush,
C_Exception,
C_Jump,
C_DBranch,
C_BeqBne,
C_JmpType,
C_JalDet,
C_ALUOpcode,
C_DstRegMux,
C_Op2SrcMux,
C_MemWrite,
C_MemAluMux,
C_RegWrite,
C_EPCSrc,
C_EPCWrite,
C_CauseWrite,
C_Cause
);

clk:   clockgen port map(CycleNum, C_Clk, C_Rst);

---------------------------------------------------------------------------

process
begin  -- process


-- 
--     o  o
--      U
--     \__/
--

wait for 10000 ns;

end process;
end struct;



